extern void AbortGame();
extern void MainMenu();
extern void ConfMenu();
extern void ScoreMenu();
extern void InitMainMenu();
extern void InitConfMenu();
extern void InitScoreMenu();
extern void MainMenuItemSelected();
extern void ConfMenuItemSelected();
extern void ScoreMenuItemSelected();
extern void TextEntry();
extern void ScoreEntry();
extern void IntTextEntry();
extern void SetText(char *TextId, char *Text);
extern void SetupMainMenu();
extern void SetupConfMenu();
extern void SetupScoreMenu();


extern int MainMenuItemHighlighted;
extern int ConfMenuItemHighlighted;
extern int ScoreMenuItemHighlighted;
extern char *KeyValue;
extern char KeyName[MAX_NAME_LENGTH];
extern char *TextValue;
extern char TextName[MAX_NAME_LENGTH];
extern int MaxTextLength;
extern int *IntValue;
extern int NewScreenWidth, NewScreenHeight;
extern int NumMainMenuItems;
extern int NumConfMenuItems;
extern int NumScoreMenuItems;
